# Preserving Zero-shot Capability in Supervised Fine-tuning for Multi-label Text Classification

This is the implementation of our work:
> [Preserving Zero-shot Capability in Supervised Fine-tuning for Multi-label Text Classification](https://www.csie.ntu.edu.tw/~cjlin/papers/zero_shot_one_side_tuning/zero_shot_one_side_tuning.pdf) <br>
> Si-An Chen, Hsuan-Tien Lin, Chih-Jen Lin <br>
> NAACL 2025

## Environment
The code is testsed with Python 3.10.14. Please install required packages by the following command.
```
pip install -r requirements.txt
```

## Data

Please follow `data_proc_eurlex.ipynb` to download and process the EURLEX57K dataset.

The codebase assumes following data structure: <br>
<pre>
Datasets/
└── EURLEX57K # Dataset name
    ├── raw
    │   ├── trn_X.txt # train input file, ith line is the text input for ith train data point
    │   ├── tst_X.txt # test input file, ith line is the text input for ith test data point
    │   └── Y.txt # label input file, ith line is the text input for ith label in the dataset
    ├── lbl_embs_sfr.npy # label description embeddings encoded by SFR(stored in numpy format), num_labels x num_features
    ├── Y.trn.npz # train relevance matrix (stored in scipy sparse npz format), num_train x num_labels
    └── Y.tst.npz # test relevance matrix (stored in scipy sparse npz format), num_test x num_labels
</pre>
Before running the training/testing the default code expects you to convert the input features to BERT's (or any text transformer) tokenized input indices. You can achieve that by running:
```shell
dataset="EURLEX57K"
python utils/tokenization_utils.py --data-path Datasets/${dataset}/raw/Y.txt --tf-max-len 512 --tf-token-type bert-base-uncased
python utils/tokenization_utils.py --data-path Datasets/${dataset}/raw/trn_X.txt --tf-max-len 512 --tf-token-type bert-base-uncased
python utils/tokenization_utils.py --data-path Datasets/${dataset}/raw/tst_X.txt --tf-max-len 512 --tf-token-type bert-base-uncased
```

## Training
Training code assumes all hyperparameter and runtime arguments are specified in a config yaml file. Please see `configs/dual_encoder.yaml` for a brief description of all parameters (you can keep most of the parameters same across experiments). See `configs/EURLEX57K/dist-asym-wde-all_softmax.yaml` or `configs/EURLEX57K/dist-asym-zslwan_softmax.yaml` to see some of the important hyperparameters that you may want to change for different experiments.

### OF-DE

```shell
# Single GPU
python train.py configs/EURLEX57k/dist-asym-wde-all_softmax.yaml

# Multi GPU
num_gpus=4
accelerate launch --config_file configs/accelerate.yaml --num_processes ${num_gpus} train.py configs/EURLEX57K/dist-asym-wde-all_softmax.yaml
```

### OF-LWAN

```shell
# Single GPU
python train.py configs/EURLEX57k/dist-asym-zslwan_softmax.yaml

# Multi GPU
num_gpus=4
accelerate launch --config_file configs/accelerate.yaml --num_processes ${num_gpus} train.py configs/EURLEX57K/dist-asym-zslwan_softmax.yaml
```

## Reference
- DEXML: https://github.com/nilesh2797/DEXML